#include <assert.h>
#include <fcntl.h>
#include <stddef.h>
#include <unistd.h>

#include <sys/ioctl.h>

#include "spi.h"

int spi_open(const struct driver *drv)
{
    assert(drv != NULL);

    int res;
    struct spi *this = (struct spi *)(drv->dev);

    // open spi dev
    this->spi_file = open(this->spi_device, O_RDWR);
    if(this->spi_file < 0) {
        return this->spi_file;
    }

    // set spi mode
    res = ioctl(this->spi_file, SPI_IOC_WR_MODE, &(this->mode));
    if(res < 0) {
        close(this->spi_file);
        return res;
    }
    res = ioctl(this->spi_file, SPI_IOC_RD_MODE, &(this->mode));
    if(res < 0) {
        close(this->spi_file);
        return res;
    }

    // set word length
    res = ioctl(this->spi_file, SPI_IOC_WR_BITS_PER_WORD, &(this->bits_per_word));
    if(res < 0) {
        close(this->spi_file);
        return res;
    }
    res = ioctl(this->spi_file, SPI_IOC_RD_BITS_PER_WORD, &(this->bits_per_word));
    if(res < 0) {
        close(this->spi_file);
        return res;
    }

    // set max speed
    res = ioctl(this->spi_file, SPI_IOC_WR_MAX_SPEED_HZ, &(this->speed));
    if(res < 0) {
        close(this->spi_file);
        return res;
    }
    res = ioctl(this->spi_file, SPI_IOC_RD_MAX_SPEED_HZ, &(this->speed));
    if(res < 0) {
        close(this->spi_file);
        return res;
    }

    this->is_open = true;
    return res;
}

int spi_close(const struct driver *drv)
{
    assert(drv != NULL);

    int res = -1;
    struct spi *this = (struct spi *)(drv->dev);

    if(this->is_open == true) {
        close(this->spi_file);
        this->is_open = false;
        res = 0;
    }
    return res;
}

int spi_read(const struct driver *drv, char *buffer, unsigned int len)
{
    assert(drv != NULL);

    struct spi *this = (struct spi *)(drv->dev);
    struct spi_ioc_transfer tr = {
        .tx_buf = (unsigned long)0,
        .rx_buf = (unsigned long)buffer,
        .delay_usecs = 0,
        .speed_hz = this->speed,
        .bits_per_word = this->bits_per_word,
        .len = len,
    };

    int res = ioctl(this->spi_file, SPI_IOC_MESSAGE(1), &tr);

    return res;
}

int spi_write(const struct driver *drv, const char *buffer, unsigned int len)
{
    assert(drv != NULL);

    struct spi *this = (struct spi *)(drv->dev);
    struct spi_ioc_transfer tr = {
        .tx_buf = (unsigned long)buffer,
        .rx_buf = (unsigned long)0,
        .len = len,
        .delay_usecs = 0,
        .speed_hz = this->speed,
        .bits_per_word = this->bits_per_word,
    };

    int res = ioctl(this->spi_file, SPI_IOC_MESSAGE(1), &tr);

    return res;
}

int spi_ioctl(const struct driver *drv, unsigned int cmd, unsigned int argc, va_list args)
{
    assert(drv != NULL);
    return 0;
}
